@@ -18,7 +18,9 @@ def default(val, d):
 def cache_fn(f):
     cache = None
     @wraps(f)
-    def cached_fn(*args, **kwargs):
+    def cached_fn(*args, _cache = True, **kwargs):
+        if not _cache:
+            return f(*args, **kwargs)
         nonlocal cache
         if cache is not None:
             return cache
@@ -40,22 +42,11 @@ def fourier_encode(x, max_freq, num_bands = 4, base = 2):
 
 # helper classes
 
-class RMSNorm(nn.Module):
-    def __init__(self, dim, eps = 1e-5):
-        super().__init__()
-        self.scale = dim ** -0.5
-        self.g = nn.Parameter(torch.ones(1, 1, dim))
-        self.eps = eps
-
-    def forward(self, x):
-        norm = torch.norm(x, dim=-1, keepdim=True) * self.scale
-        return x * self.g / norm.clamp(min=self.eps)
-
 class PreNorm(nn.Module):
     def __init__(self, dim, fn, context_dim = None):
         super().__init__()
         self.fn = fn
-        self.norm = RMSNorm(dim)
+        self.norm = nn.LayerNorm(dim)
         self.norm_context = nn.LayerNorm(context_dim) if exists(context_dim) else None
 
     def forward(self, x, **kwargs):
@@ -87,7 +78,7 @@ class FeedForward(nn.Module):
         return self.net(x)
 
 class Attention(nn.Module):
-    def __init__(self, query_dim, context_dim = None, heads = 8, dim_head = 75, dropout = 0.):
+    def __init__(self, query_dim, context_dim = None, heads = 8, dim_head = 64, dropout = 0.):
         super().__init__()
         inner_dim = dim_head * heads
         context_dim = default(context_dim, query_dim)
@@ -165,20 +156,22 @@ class Perceiver(nn.Module):
         get_latent_attn = lambda: PreNorm(latent_dim, Attention(latent_dim, heads = latent_heads, dim_head = latent_dim_head, dropout = attn_dropout))
         get_latent_ff = lambda: PreNorm(latent_dim, FeedForward(latent_dim, dropout = ff_dropout))
 
-        if weight_tie_layers:
-            get_cross_attn, get_cross_ff, get_latent_attn, get_latent_ff = map(cache_fn, (get_cross_attn, get_cross_ff, get_latent_attn, get_latent_ff))
+        get_cross_attn, get_cross_ff, get_latent_attn, get_latent_ff = map(cache_fn, (get_cross_attn, get_cross_ff, get_latent_attn, get_latent_ff))
 
         self.layers = nn.ModuleList([])
-        for _ in range(depth):
+        for i in range(depth):
+            should_cache = i > 1 and weight_tie_layers
+            cache_args = {'_cache': should_cache}
+
             self.layers.append(nn.ModuleList([
-                get_cross_attn(),
-                get_cross_ff(),
-                get_latent_attn(),
-                get_latent_ff()
+                get_cross_attn(**cache_args),
+                get_cross_ff(**cache_args),
+                get_latent_attn(**cache_args),
+                get_latent_ff(**cache_args)
             ]))
 
         self.to_logits = nn.Sequential(
-            RMSNorm(latent_dim),
+            nn.LayerNorm(latent_dim),
             nn.Linear(latent_dim, num_classes)
         )
 
